import { useCallback, useRef, useReducer, useLayoutEffect } from 'react';

function shallowEqual(objA, objB) {
  if (Object.is(objA, objB)) {
    return true;
  }

  if (typeof objA !== 'object' || objA === null || typeof objB !== 'object' || objB === null) {
    return false;
  }

  const keysA = Object.keys(objA);

  if (keysA.length !== Object.keys(objB).length) {
    return false;
  }

  for (let i = 0; i < keysA.length; i++) {
    if (!Object.prototype.hasOwnProperty.call(objB, keysA[i]) || !Object.is(objA[keysA[i]], objB[keysA[i]])) {
      return false;
    }
  }

  return true;
}

const reducer = (state, newState) => newState;

function create(createState) {
  const listeners = new Set();

  const setState = partialState => {
    state = Object.assign({}, state, typeof partialState === 'function' ? partialState(state) : partialState);
    listeners.forEach(listener => listener(state));
  };

  const getState = () => state;

  const subscribe = listener => {
    listeners.add(listener);
    return () => {
      listeners.delete(listener);
    };
  };

  const destroy = () => {
    listeners.clear();
    state = {};
  };

  function useStore(selector, dependencies) {
    // State selector gets entire state if no selector was passed in
    const stateSelector = typeof selector === 'function' ? selector : getState;
    const selectState = useCallback(stateSelector, dependencies);
    const selectStateRef = useRef(selectState);
    let [stateSlice, dispatch] = useReducer(reducer, state, selectState); // Call new selector if it has changed

    if (selectState !== selectStateRef.current) stateSlice = selectState(state); // Store in ref to enable updating without rerunning subscribe/unsubscribe

    const stateSliceRef = useRef(stateSlice); // Update refs only after view has been updated

    useLayoutEffect(() => {
      selectStateRef.current = selectState;
      stateSliceRef.current = stateSlice;
    }, [selectState, stateSlice]); // Subscribe/unsubscribe to the store only on mount/unmount

    useLayoutEffect(() => {
      return subscribe(() => {
        // Use the last selector passed to useStore to get current state slice
        const selectedSlice = selectStateRef.current(state); // Shallow compare previous state slice with current and rerender only if changed

        if (!shallowEqual(stateSliceRef.current, selectedSlice)) dispatch(selectedSlice);
      });
    }, []);
    return stateSlice;
  }

  let state = createState(setState, getState);
  const api = {
    destroy,
    getState,
    setState,
    subscribe
  };
  return [useStore, api];
}

export default create;
